/* zproto.c for X680x0 */

#include	"zaddin.h"
#include	"iofunc.h"
#include	"oproto.h"

#define		PACKET_SIZE	(1024*4)

unsigned short
RecvWord()
{
	unsigned char	buf[2];
	RecvData( 2, buf );
	return	(buf[1]<<8)+buf[0];
}

Recv1byte()
{
	unsigned char	c;
	OptRecvOpen();
	RecvData( 1, &c );
	return	c;
}

Send1byte( code )
{
	unsigned char	c= code;
	OptSendOpen();
	SendData( 1, &c );
	OptSendFlush();
}

WaitCode( code )
unsigned char	code;
{
	for(;;){
		unsigned char	c= Recv1byte();
		if( c == code )
			return	TRUE;
		else if( c == cCAN )
			return	TRUE;
	}
}

RecvPacket( packet, num )
unsigned char	*packet, num;
{
	int	length;
	OptRecvOpen();
	RecvData( 1, packet );
	if( *packet == cET ){	/* end of data */
		return	ENDOF;
	}else if( *packet != 0x01 ){
		Error( "packet format error", "" );
		return	RETRY;
	}
	RecvData( 2, packet );
	if( packet[0] != num || packet[1] != ((~num)&0xff) ){
		Error( "packet number error", "" );
		return	RETRY;
	}

	length= RecvWord();
	RecvData( length, packet );
	if( RecvWord() != CheckSum( length, packet ) ){
		Error( "check sum error", "" );
		return	RETRY;
	}
	return	length;
}

SendPacket( length, num, packet )
unsigned char	num,
		*packet;
{
	static unsigned char	buf[6]= { 0x01, 0x01, 0xfe, 0, 0, 0 };
	unsigned short	check_sum;
	OptSendOpen();
	buf[1]= num;
	buf[2]= ~num;
	buf[3]= length;
	buf[4]= length>>8;
	SendData( 5, buf );
	SendData( length, packet );
	check_sum= CheckSum( length, packet );
	buf[3]= check_sum;
	buf[4]= check_sum>>8;
	SendData( 2, buf+3 );
	OptSendFlush();
	return	TRUE;
}

RecvCommand( data )
unsigned char	*data;
{
	Send1byte( cENQ );
	RecvPacket( data, 1 );
	Send1byte( cACK );

	WaitCode( cET );
	Send1byte( cACK );
}

SendCommand( length, data )
unsigned char	*data;
{
	do{
		SendPacket( length, 1, data );
	}while( Recv1byte() == cNACK );
	do{
		Send1byte( cET );
	}while( Recv1byte() == cNACK );
}

RecvFileData( buflen, data, fname )
unsigned char	*data, *fname;
{
	int		length= 0, i= 1;
	unsigned char	packet_buf[PACKET_SIZE];

	SendCommand( 3, "CFV" );
	RecvCommand( packet_buf );
	if( n_fstrcmp( "DOK", packet_buf ) ){
		Error( "Zaurus ڑ܂͑ҋ@Ԃɂ܂", "" );
		return	FATAL;
	}
	WaitCode( cENQ );

	s_strcpy( s_strcpy( packet_buf, "RFV000S1:" ), fname );
	packet_buf[3]= buflen;
	packet_buf[4]= buflen>>8;
	packet_buf[5]= buflen>>16;
	SendCommand( n_strlen(fname)+10, packet_buf );

	Send1byte( cENQ );
	for(;;){
		int	size= RecvPacket( packet_buf, i );
Print( "o" );
		if( size == RETRY ){
			Send1byte( cNACK );
			continue;
		}
		i++;
		Send1byte( cACK );
		if( size >= 3 ){
			if( !n_fstrcmp( "ANG", packet_buf ) ){
				Error( fname, " file not found" );
				return	FATAL;
			}else if( n_fstrcmp( "AFV", packet_buf ) ){
				Error( "packet error", "" );
				break;
			}
			length+= (size-= 3);
			s_memcpy_simple( size, data, packet_buf+3 );
			data+= size;
			continue;
		}
		break;
	}
Error( " end","" );
	return	length;
}

#define		DTP_SIZE	(256-3)
SendFileData( length, data, fname )
unsigned char	*data, *fname;
{
	int		i= 1;
	unsigned char	packet_buf[PACKET_SIZE];
	SendCommand( 3, "CFD" );
	RecvCommand( packet_buf );
	if( n_fstrcmp( "DOK", packet_buf ) ){
		Error( "Zaurus ڑ܂͑ҋ@Ԃɂ܂", "" );
		return	FATAL;
	}
	WaitCode( cENQ );
	s_strcpy( s_strcpy( packet_buf, "RFDS1:" ), fname );
	do{
		SendPacket( n_strlen(fname)+7, i++, packet_buf );
	}while( Recv1byte() == cNACK );
	do{
Print( "o" );
		s_strcpy( packet_buf, "RFD" );
		s_memcpy_simple( DTP_SIZE, packet_buf+3, data );
		data+= DTP_SIZE;
		do{
			SendPacket( length >= DTP_SIZE ? DTP_SIZE+3 : length+3,
						i++, packet_buf );
		}while( Recv1byte() == cNACK );
		length-= DTP_SIZE;
	}while( length >= 0 );
	do{
		Send1byte( cET );
	}while( Recv1byte() != cACK );

	RecvCommand( packet_buf );
	if( n_fstrcmp( "AEX", packet_buf ) ){
		Error( fname, ":MɎs܂" );
		return	FATAL;
	}
Error( " end","" );
	return	TRUE;
}

ResetZaurus()
{
	unsigned char	packet_buf[PACKET_SIZE];
	SendCommand( 3, "CLE" );
	RecvCommand( packet_buf );
	if( n_fstrcmp( "DOK", packet_buf ) ){
		Error( "ZbgɎs܂", "" );
		return	FALSE;
	}
}

ExitZaurus()
{
	WaitCode( cENQ );
}
